@@ -175,11 +175,11 @@ class clientHTTPSConnection (httplib.HTTPSConnection):
                 transport = HTTPSClientCertTransport(None,None, self.cert_path))
         cert = client.service.get_ca()
         if cert == '1':
-            print _('Invalid server certificate')
+            print _("Invalid server certificate!")
             sys.exit()
             
         if cert == '2':
-            print _('CA not found on the server')
+            print _("CA certificate not found on the server")
             sys.exit()
             
         try:
@@ -289,7 +289,7 @@ class clientHTTPSConnection (httplib.HTTPSConnection):
             print "%s : %s" %(item[0], item[1])
         
         print '\n'+_('Add this server certificate to trusted (s) or')
-        print _('Try to add the CA and ROOT certificates to trusted (c) or')
+        print _('Try to add the CA and root certificates to trusted (c) or')
         choice = raw_input (_("Quit (q)? s/c/[q]: "))
         if choice.lower() in ['s', 'c']:
             #self.sock = ssl.wrap_socket(sock)
@@ -336,7 +336,7 @@ class clientHTTPSConnection (httplib.HTTPSConnection):
                     print _("Error creating directory %s") %root_cert_dir
                     sys.exit()
             
-            print '\n'+_("Try to add the CA and ROOT certificates")
+            print '\n'+_("Add the CA and root certificates")
             self.list_ca_certs = []
             self.add_ca_cert(cert, self.list_ca_certs)
             sys.exit()
