@@ -26,6 +26,7 @@ class DataVarsCore(DataVars):
         """
         Import lib and core variables
         """
-        self.importData()
-        self.importData('calculate.core.variables')
+        self.importVariables()
+        self.importVariables('calculate.core.variables')
+        self.defaultModule = "core"
 
