@@ -17,18 +17,19 @@
 ###############################################################################
 ################ Job with Command Line and Certificates #######################
 ###############################################################################
-import os, glob, socket
+import os, glob, socket,re
 import subprocess
 import hashlib
 import importlib,sys
 from func import new_key_req, uniq
 from calculate.lib.utils import ip as ip
+from calculate.lib.utils.text import getTerminalSize
 from calculate.lib.datavars import VariableError
 from calculate.core.datavars import DataVarsCore
 from itertools import repeat
 
 from calculate.lib.cl_lang import setLocalTranslate
-setLocalTranslate('cl_core',sys.modules[__name__])
+setLocalTranslate('cl_core3',sys.modules[__name__])
 
 def getIpLocal():
     for interface in ip.getInterfaces():
@@ -98,7 +99,7 @@ def check_server_certificate(cert, key, cert_path, args, port, auto = False):
                                 %(host_name, port)) 
             if name.L in ['', None]:
                 name.L = host_name + ':' + str(port)
-            name.ST = raw_input (_('State name: '))
+            name.ST = raw_input (_('City: '))
             if not name.ST:
                 name.ST = ''
             else:
@@ -143,7 +144,7 @@ def check_server_certificate(cert, key, cert_path, args, port, auto = False):
         try:
             fd.write(open(cert_path+'/root.crt', 'r').read())
         except:
-            print _('error writing to (reading) files from directory %s') \
+            print _('error writing to (reading from) files in directory %s') \
                                                                 %cert_path
         fd.close()
         print _("OK")
@@ -192,7 +193,7 @@ def check_server_certificate(cert, key, cert_path, args, port, auto = False):
         serv_host_name = client.service.get_server_host_name()
 
         if os.path.exists(key) and os.path.exists(cert_path + '/server.csr'):
-            print _('private key and request now exist')
+            print _("the private key and request now exist")
             ask = raw_input(_("Create a new private key and request?")+\
                             " y/[n]: ")
             if ask.lower() in ['y','yes']:
@@ -248,12 +249,11 @@ def check_server_certificate(cert, key, cert_path, args, port, auto = False):
             print _('The signature request was rejected!')
             return 1
         elif cert == '2':
-            print _("The signature request has not been reviewed yet.")
+            print _("The signature request has not been examined yet.")
             print _("Your request ID = %s") %req_id
             return 1
         elif cert == '3':
-            print _('The request for signature does not match the '
-                    'earlier data.')
+            print _('The signature request does not match earlier data.')
             return 1
         elif cert == '4':
             print _("The request was sent from another IP.")
@@ -272,7 +272,7 @@ def check_server_certificate(cert, key, cert_path, args, port, auto = False):
         fd.close()
         return 0
 
-def create_path(data_path, certbase, rights, group_rights):
+def create_path(data_path, certbase, rights, group_rights,local_data_path):
     if not os.path.exists(certbase):
         if not os.path.exists(data_path+'/client_certs'):
             try:
@@ -289,6 +289,12 @@ def create_path(data_path, certbase, rights, group_rights):
         except OSError:
             print _("cannot create directory %s") %(data_path+'/conf')
 
+    if not os.path.exists(local_data_path+'/conf'):
+        try:
+            os.makedirs(local_data_path+'/conf')
+        except OSError:
+            print _("cannot create directory %s") %(local_data_path+'/conf')
+
     if not os.path.isfile(rights):
         rights_text = '# example of content:\n'+ \
                       '# certificate number 2 has right to run method ' + \
@@ -1103,101 +1109,8 @@ def revoke_signed_cert(revoke_cert_id, data_path, cert_path):
     cmd_show_crl = "openssl crl -text -noout -in %s" %CRL
     os.system(cmd_show_crl)
 
-def import_datavars(variable):
-    mapName = {'main':'lib'}
-    mapMethod = {'importLib':'importData'}
-    mapObject = {'DataVarsLib':'DataVars'}
-    if "." in variable:
-        moduleName, varName = variable.split('.')
-        if moduleName[0] == '!':
-            moduleName = moduleName[1:]
-        moduleName = mapName.get(moduleName,moduleName)
-    else:
-        varName = variable
-        moduleName = "lib"
-    if "[" in varName:
-        varName,op,indexVal = varName.partition('[')
-        indexVal = int(indexVal.strip(']'))
-    else:
-        indexVal = None
-    importMethod = "import%s"%moduleName.capitalize()
-    importMethod = mapMethod.get(importMethod,importMethod)
-    mod = importlib.import_module("calculate.%s.datavars"%moduleName)
-    objName = "DataVars%s"%moduleName.capitalize()
-    obj = getattr(mod,mapObject.get(objName,objName))()
-    getattr(obj,importMethod)()
-    obj.flIniFile()
-    return obj,moduleName,varName,indexVal
-
-def write_variable(write_variable):
-    try:
-        variable,op,value = write_variable.partition('=')
-        obj,moduleName,varName,indexVal = import_datavars(variable)
-        if variable[0] == "!":
-            obj.Delete(varName.lstrip("!"),header=moduleName)
-            return
-        if indexVal:
-            raise VariableError(_("You should not specify the value index."))
-        if "list" in obj.getInfo(varName).type:
-            obj.Write(varName,value.split(','))
-    except Exception as e:
-        sys.stderr.write(str(e)+"\n")
-        sys.exit(1)
-
-def append_variable(append_variable):
-    try:
-        variable,op,value = append_variable.partition('=')
-        obj,moduleName,varName,indexVal = import_datavars(variable)
-        if variable[0] == "!":
-            variable = variable[1:]
-        if not "list" in obj.getInfo(variable).type:
-            raise VariableError(
-                _("--append-variable works with list variables only"))
-        value = value.split(',')
-        if varName[0] == "!":
-            value = list(set(obj.Get(variable))-set(value))
-        else:
-            value = list(set(obj.Get(variable))|set(value))
-        obj.Write(variable,filter(None,value))
-    except Exception as e:
-        sys.stderr.write(str(e)+"\n")
-        sys.exit(1)
-
-def print_variable(variable,human_readable):
-    isListOrTuple = lambda x: type(x) in (list,tuple)
-    try:
-        obj,moduleName,varName,indexVal = import_datavars(variable)
-        varVal = obj.Get(varName,humanreadable=human_readable)
-        if human_readable:
-            sys.stdout.write("%s: "%(obj.getInfo(varName).label or varName))
-            if obj.getInfo(varName).type == "table":
-                print 
-                print ",".join([varLabel 
-                      for varLabel in map(lambda x:obj.getInfo(x).label or x,
-                      obj.getInfo(varName).source)])
-        if isListOrTuple(varVal):
-            if indexVal is None:
-                if varVal and isListOrTuple(varVal[0]):
-                    listValue = varVal
-                else:
-                    listValue = repeat(varVal,1)
-                for val in listValue:
-                    print ",".join(val)
-            elif indexVal < len(varVal):
-                if isListOrTuple(varVal[indexVal]):
-                    print ",".join(varVal[indexVal])
-                else:
-                    print varVal[indexVal]
-        else:
-            if not indexVal is None:
-                raise VariableError(_("This variable is not a list"))
-            print varVal
-    except Exception as e:
-        sys.stderr.write(str(e)+"\n")
-        sys.exit(1)
-
 #################################################################
-def parse():
+def parse(full=False):
     import argparse
     from methods_func import RawAndDefaultsHelpFormatter
     parser = argparse.ArgumentParser(add_help=False,
@@ -1205,79 +1118,81 @@ def parse():
     parser.add_argument(
         '-h', '--help', action='store_true', default=False,
         dest='help', help=_("show this help message and exit"))
-    parser.add_argument(
-        '--start',  action='store_true', default=False, dest = 'start',
-        help=_('server started'))
-    parser.add_argument(
-        '-p', '--port', type=int, default = '8888', dest='port',
-        help=_('port number'))
-    parser.add_argument(
-        '--pid-file', type=str, dest='pidfile', 
-        help=_('Specify the location of the PID file'))
-    parser.add_argument(
-        '-c', '--cert', type=str, dest='Id', 
-        help=_('operations with certificates (number or "all"). Server not'
-               ' running.'))
-    parser.add_argument(
-        '-r', '--show-request', type=str, dest='req_id', 
-        help=_("view client requests (number or \"all\")"))
-    parser.add_argument(
-        '--sc', '--server-cert', type=str, dest='cert_id',
-        help=_('view servers certificates (number or "all"). '
-               'Server not running.'))
-    parser.add_argument(
-        '-b', '--bootstrap',  type=str, dest='bootstrap_user_name',
-        help=_('bootstrap action'))
-    parser.add_argument(
-        '--remove-certs',  action='store_true', default=False,
-        dest = 'remove_certificates', help=_('use with --bootstrap to remove '
-        'all certificates, requests and config files on the server'))
-    parser.add_argument(
-        '-d', '--dump',  action='store_true', default=False, dest = 'dump',
-        help=_('dump (use with -c [ID])'))
+    if full:
+        parser.add_argument(
+            '--start',  action='store_true', default=False, dest = 'start',
+            help=_('server started'))
+        parser.add_argument(
+            '-p', '--port', type=int, default = '8888', dest='port',
+            help=_('port number'))
+        parser.add_argument(
+            '--pid-file', type=str, dest='pidfile', 
+            help=_('Specify the location of the PID file'))
+        parser.add_argument(
+            '-c', '--cert', type=str, dest='Id', 
+            help=_('operations with certificates (number or "all"). Server not'
+                   ' running.'))
+        parser.add_argument(
+            '-r', '--show-request', type=str, dest='req_id', 
+            help=_("view client requests (number or \"all\")"))
+        parser.add_argument(
+            '--sc', '--server-cert', type=str, dest='cert_id',
+            help=_('view servers certificates (number or "all"). '
+                   'Server not running.'))
+        parser.add_argument(
+            '-b', '--bootstrap',  type=str, dest='bootstrap_user_name',
+            help=_('bootstrap action'))
+        parser.add_argument(
+            '--remove-certs',  action='store_true', default=False,
+            dest = 'remove_certificates', help=_('use with option --bootstrap to remove '
+            'all certificates, requests and config files on the server'))
+        parser.add_argument(
+            '-d', '--dump',  action='store_true', default=False, dest = 'dump',
+            help=_('dump (use with option -c [ID])'))
     parser.add_argument(
         '--db','--debug',  action='store_true', default=False, dest = 'debug',
         help=_('debug'))
-    parser.add_argument(
-        '--rm','--remove',  action='store_true', default=False, dest='remove',
-        help=_('remove the selected certificate (use with -c [ID])'))
-    parser.add_argument(
-        '--right-add', type=str, dest='right_add',
-        help=_('add permissions for a certificate (or certificates, '
-               'comma-separated) (use with -c [ID])'))
-    parser.add_argument(
-        '--right-del', type=str, dest='right_del',
-        help=_('remove permissions for a certificate (use with -c [ID])'))
-    parser.add_argument(
-        '--gen-root-cert',  action='store_true', default=False,
-        dest = 'gen_root_cert', help=_('generate a root (CA) certificate'))
-    parser.add_argument(
-        '--gen-cert-by', type=str, dest='host',
-        help=_('send a request to sign the root certificate'))
-    parser.add_argument(
-        '--get-cert-from', type=str, dest='root_host',
-        help=_('fetch the signed certificate from the server'))
-    parser.add_argument(
-        '--use-root-as-server',  action='store_true', default=False,
-        dest = 'use_root_cert', help=_('use the root certificate as the '
-                                       'server certificate'))
-    parser.add_argument(
-        '--sign-client', type=str, dest='id_client_req',
-        help=_("sign the client's request with the server certificate"))
-    parser.add_argument(
-        '--sign-server', type=str, dest='id_server_req',
-        help=_("sign the server's request with the root certificate"))
-    parser.add_argument(
-        '--del-server-req', type=str, dest='id_del_req',
-        metavar = 'id_req', help=_("refuse to sign the certificate at "
-                                   "server's request"))
-    parser.add_argument(
-        '--del-client-req', type=str, dest='id_del_client_req',
-        metavar = 'id_req', help=_("decline client's request for signature"))
-    parser.add_argument(
-        '--rv', '--revoke-cert', type=str, dest='revoke_cert_id', 
-        help=_('revoke an earlier signed server (or ca) certificate. rm to '
-               'remove the CRL'))
+    if full:
+        parser.add_argument(
+            '--rm','--remove',  action='store_true', default=False, dest='remove',
+            help=_('remove the selected certificate (use with option -c [ID])'))
+        parser.add_argument(
+            '--right-add', type=str, dest='right_add',
+            help=_('add permissions for a certificate (or certificates, '
+                   'comma-separated) (use with option -c [ID])'))
+        parser.add_argument(
+            '--right-del', type=str, dest='right_del',
+            help=_('remove permissions for a certificate (use with option -c [ID])'))
+        parser.add_argument(
+            '--gen-root-cert',  action='store_true', default=False,
+            dest = 'gen_root_cert', help=_('generate a root (or CA) certificate'))
+        parser.add_argument(
+            '--gen-cert-by', type=str, dest='host',
+            help=_("send a signature request for the root certificate"))
+        parser.add_argument(
+            '--get-cert-from', type=str, dest='root_host',
+            help=_('fetch the signed certificate from the server'))
+        parser.add_argument(
+            '--use-root-as-server',  action='store_true', default=False,
+            dest = 'use_root_cert', help=_('use the root certificate as the '
+                                           'server certificate'))
+        parser.add_argument(
+            '--sign-client', type=str, dest='id_client_req',
+            help=_("sign the client's request with the server certificate"))
+        parser.add_argument(
+            '--sign-server', type=str, dest='id_server_req',
+            help=_("sign the server's request with the root certificate"))
+        parser.add_argument(
+            '--del-server-req', type=str, dest='id_del_req',
+            metavar = 'id_req', help=_("refuse to sign the certificate at "
+                                       "server's request"))
+        parser.add_argument(
+            '--del-client-req', type=str, dest='id_del_client_req',
+            metavar = 'id_req', help=_("decline client's request for signature"))
+        parser.add_argument(
+            '--rv', '--revoke-cert', type=str, dest='revoke_cert_id', 
+            help=_('revoke an earlier signed server (or CA) certificate. Run rm to '
+                   'remove the CRL'))
     parser.add_argument(
         '--method', type=str, dest='method', help=_('call method'))
     parser.add_argument(
@@ -1286,34 +1201,26 @@ def parse():
     parser.add_argument(
         '--no-progress', action='store_true', default=False, 
         dest = 'no_progress', help=_('do not display the progress bar'))
+    parser.add_argument(
+        '--gui-progress', action='store_true', default=False, 
+        dest = 'gui_progress', help=_("display the GUI progress bar"))
+    parser.add_argument(
+        '--gui-warning', action='store_true', default=False, 
+        dest = 'gui_warning', help=_('display warnings at the end'))
     parser.add_argument(
         '-f', '--force', action='store_true', default=False,
         dest = 'no_questions', help=_('silent during the process'))
     parser.add_argument(
         '-P', action='store_true', default=False,
         dest = 'stdin_passwd',
-        help=_('use passwords for the users accounts from standard input'))
-    parser.add_argument(
-        '--variable', type=str, dest='variable', 
-        help=_("display the value for the specified variable "
-               "(section.variablename). If section skipped then use "
-               "variables from the main section"))
-    parser.add_argument(
-        '--human-readable', action='store_true', default=False,
-        dest='human_readable',
-        help=_("display the variable as human readable"))
-    parser.add_argument(
-        '--write-variable', type=str, dest='write_variable', 
-        help=_("write a value for the variable (section.variablename=value)"))
-    parser.add_argument(
-        '--append-variable', type=str, dest='append_variable', 
-        help=_("append value to list variable (section.variablename=value)"))
-    parser.add_argument(
-        '--check', action='store_true', default=False, dest = 'check',
-        help=_('configuration check'))
-    parser.add_argument(
-        '--create-symlink', action='store_true', default=False,
-        dest='create_symlink', help=_("Create symlinks for methods"))
+        help=_('use passwords from standard input for users accounts'))
+    if full:
+        parser.add_argument(
+            '--check', action='store_true', default=False, dest = 'check',
+            help=_('configuration check'))
+        parser.add_argument(
+            '--create-symlink', action='store_true', default=False,
+            dest='create_symlink', help=_("Create symlinks for methods"))
     parser.add_argument(
         '--log-path', type=str, dest='log_path',
         help=_('path to log files'))
