@@ -26,7 +26,7 @@ from calculate.core.server.api_types import (Field, GroupField,
 from func import uniq
 from calculate.lib.datavars import VariableError
 from calculate.lib.cl_lang import getLazyLocalTranslate, setLocalTranslate
-setLocalTranslate('cl_core',sys.modules[__name__])
+setLocalTranslate('cl_core3',sys.modules[__name__])
 
 from calculate.core import datavars
 import traceback
@@ -236,7 +236,7 @@ class CoreWsdl:
         dv.addGroup(_("Certificates"),
             normal=('cl_page_count','cl_page_offset'),
             next_label=_("Next"))
-        view = ViewInfo(dv)
+        view = ViewInfo(dv,viewparams=params)
         self.set_cache(sid, 'core_show_certs', "vars", dv, smart=False)
         return view
 
@@ -277,7 +277,7 @@ class CoreWsdl:
         dv.addGroup(None,
             normal=('cl_cert_id', 'cl_cert_groups', 'cl_cert_perms'))
 
-        view = ViewInfo(dv)
+        view = ViewInfo(dv,viewparams=params)
 
         group = GroupField(name='',last=True)
         group.fields = []
@@ -392,7 +392,7 @@ class CoreWsdl:
 #                                    field = 'cl_group_name')]
 #        if info.cl_group_rights == None:
 #            return [ReturnedMessage(type='error', field='cl_group_rights',\
-#                    message=_('use --group-rights to change the permissions'))]
+#                    message=_("use option --group-rights to toggle permissions"))]
 #        try:
 #            dv.Set('cl_group_rights', info.cl_group_rights)
 #        except VariableError, e:
@@ -411,13 +411,13 @@ class CoreWsdl:
 #            dv.flIniFile()
 #            dv.Get('cl_group_name')
 #
-#        dv.addGroup(_("Change the group"),
+#        dv.addGroup(_("Toggle groups"),
 #            normal=('cl_group_name', 'cl_group_rights'),
 #            next_label=_("Done"))
 #
 #        view = ViewInfo(dv)
 #
-#        group = GroupField(name=_("Change the group"), nextlabel=_("Done"), \
+#        group = GroupField(name=_("Toggle groups"), nextlabel=_("Done"), \
 #                           last=True)
 #        group.fields = []
 #        group.fields.append(Field(
