@@ -17,11 +17,138 @@
 import pickle, random
 import threading
 import sys, os
+import traceback
 from traceback import print_exc
-from api_types import ReturnProgress
+from api_types import ReturnProgress,ViewParams,Integer,ViewInfo
 from decorators import Dec
-from calculate.lib.cl_lang import setLocalTranslate
-setLocalTranslate('cl_core',sys.modules[__name__])
+from calculate.lib.cl_lang import setLocalTranslate,getLazyLocalTranslate
+from soaplib.serializers.primitive import Integer
+from soaplib.service import rpc
+setLocalTranslate('cl_core3',sys.modules[__name__])
+__ = getLazyLocalTranslate(_)
+
+def commonView(self,sid,params,arg):
+    dv = self.get_cache(sid,arg,"vars")
+    if not dv:
+        dv = getattr(self,"%s_vars"%arg)()
+    else:
+        dv.processRefresh()
+    view = ViewInfo(dv,viewparams=params)
+    self.set_cache(sid, arg, "vars",dv,smart=False)
+    return view
+
+def catchExcept(*skipException):
+    class wrapper:
+        def __init__(self,f):
+            self.f = f
+            self.func_name = f.func_name
+            self.func_code = f.func_code
+            self.__doc__ = f.__doc__
+            self.__name__ = f.__name__
+        
+        def __call__(self,*args,**kwargs):
+            try:
+                return self.f(*args,**kwargs)
+            except BaseException as e:
+                from calculate.core.server.api_types import ViewInfo,GroupField,Field
+                view = ViewInfo(groups=[])
+                group = GroupField(name=_("Error"),last=True)
+                group.fields = []
+                group.fields.append(Field(
+                        name = "error",
+                        label = str(e),
+                        default = 'color:red;',
+                        element = "error"))
+                view.groups.append(group)
+                if not any(isinstance(e,x) for x in skipException):
+                    for i in apply(traceback.format_exception, sys.exc_info()):
+                        print i,
+
+                return view
+    return wrapper
+
+def shortTraceback(e1,e2,e3):
+    """
+    Return short traceback
+    """
+    frame = e3
+    for i in apply(traceback.format_exception, (e1,e2,e3)):
+        print i,
+    while(frame.tb_next):
+        frame = frame.tb_next
+    module,part = os.path.split(frame.tb_frame.f_code.co_filename)
+    if part.endswith('.py'):
+        part = part[:-3]
+    fallbackmod = part
+    modname = [part]
+    while module != '/' and not module.endswith('site-packages'):
+        module,part = os.path.split(module)
+        modname.insert(0,part)
+    if module.endswith('site-packages'):
+        modname = ".".join(modname)
+    else:
+        modname = fallbackmod
+    return "%s:%s(%s:%s)"%(e1.__name__,e2.message,modname,frame.tb_lineno)
+
+def safetyWrapper(native_errors=(Exception,),
+                  man_int=__("Manually interrupted"),
+                  post_action=lambda self,x:None,
+                  failed_message="",
+                  success_message=""):
+    """
+    Standard decorator for logical method called by wsdl
+    """
+    def wrapper(f):
+        def tmp(self,*args,**kwargs):
+            error = None
+            try:
+                try:
+                    res = f(self,*args,**kwargs)
+                except EOFError as e:
+                    error = str(e)
+                except native_errors as e:
+                    error = str(e)
+                except Exception as e:
+                    error = shortTraceback(*sys.exc_info())
+                except KeyboardInterrupt:
+                    error = str(man_int)
+                if error:
+                    self.printERROR(error)
+                try:
+                    post_action(self,error)
+                except native_errors as e:
+                    error = str(e)
+                    self.printERROR(error)
+                except KeyboardInterrupt:
+                    pass
+                if error:
+                    mess = str(failed_message)
+                    if mess:
+                        self.printERROR(str(failed_message))
+                    return False
+                mess = str(success_message)
+                if mess:
+                    self.printSUCCESS(mess)
+                self.endTask()
+                return res
+            except (BaseException,) as e:
+                error = shortTraceback(*sys.exc_info())
+                self.printERROR(error)
+                return False
+            finally:
+                try:
+                    if hasattr(self.clVars,"close"):
+                        self.clVars.close()
+                except (BaseException),e:
+                    error = ""
+                    error = shortTraceback(*sys.exc_info())
+                    self.printERROR(error)
+                    return False
+                finally:
+                    self.endFrame()
+            return res
+        return tmp
+    return wrapper
 
 class CoreWsdl():
     # client signals about presence
@@ -164,7 +291,7 @@ class CoreWsdl():
                 pickle.dump(self.process_dict, _fc)
                 _fc.close()
             except:
-                print _("Failed to write to PID file %s!") %self.PID_FILE
+                print _("Failed to read the PID file %s!") %self.PID_FILE
 
         def setProgress(self, perc, short_message = None, long_message = None):
             id = self.progress_dict['id']
@@ -211,7 +338,7 @@ class CoreWsdl():
                 id = random.randint(1, self.Num)
 
             from api_types import Table
-            table = Table(head = head, body = body, fields = fields, \
+            table = Table(head = head, body = map(lambda x:map(str,x),body), fields = fields, \
                           onClick = onClick, addAction = addAction, \
                           values = None)
             self.table_dict[id] = table
@@ -224,6 +351,9 @@ class CoreWsdl():
 
         def printSUCCESS(self, message = ''):
             self.addMessage(type = 'normal', message = message)
+        
+        def printDefault(self, message = ''):
+            self.addMessage(type = 'plain', message = message)
 
         def printWARNING(self, message):
             self.addMessage(type = 'warning', message = message)
@@ -289,8 +419,8 @@ class CoreWsdl():
         self.add_sid_pid(sid, pid)
 
         import multiprocessing
-        if not hasattr(self, 'manager'):
-            self.manager = multiprocessing.Manager()
+        if self.manager is None:
+            self.__class__.manager = multiprocessing.Manager()
         # Manager for sending glob_process_dict between watcher and process
         #manager = multiprocessing.Manager()
         self.glob_process_dict[pid] = self.manager.dict()
@@ -515,9 +645,11 @@ class CoreWsdl():
             lang = locale.locale_alias['en']
         return lang
 
-def create_symlink(data_path):
+def create_symlink(data_path,old_data_path):
     meths = Dec.conMethods
     path_to_link = '/usr/sbin'
+    path_to_user_link = '/usr/bin'
+    old_symlinks_file = os.path.join(old_data_path, 'conf/symlinks')
     symlinks_file = os.path.join(data_path, 'conf/symlinks')
     if not os.path.exists(os.path.join(data_path, 'conf')):
         try:
@@ -525,9 +657,15 @@ def create_symlink(data_path):
         except OSError:
             print _("cannot create directory %s") \
                                 %(os.path.join(data_path, 'conf'))
+    if os.path.exists(old_symlinks_file) and not os.path.exists(symlinks_file):
+        open(symlinks_file,'w').write(open(old_symlinks_file).read())
+        os.unlink(old_symlinks_file)
     fd = open(symlinks_file, 'a')
     for link in meths:
-        link_path = os.path.join(path_to_link, link)
+        if meths[link][1]:
+            link_path = os.path.join(path_to_user_link, link)
+        else:
+            link_path = os.path.join(path_to_link, link)
         if os.path.islink(link_path):
             continue
         if os.path.isfile(link_path):
@@ -545,7 +683,10 @@ def create_symlink(data_path):
     from calculate.lib.utils.files import readLinesFile
     temp_text_file = ''
     for line in readLinesFile(symlinks_file):
-        if not os.path.basename(line) in meths.keys():
+        cmdname = os.path.basename(line)
+        if not cmdname in meths.keys() or \
+            line.startswith(path_to_link) and meths[cmdname][1] or \
+            line.startswith(path_to_user_link) and not meths[cmdname][1]:
             if os.path.islink(line):
                 os.unlink(line)
                 print _('Symlink %s deleted') %line
