@@ -16,17 +16,61 @@
 #  limitations under the License.
 
 import sys, os
+import argparse
+import re
 from calculate.lib.utils.common import getpass
 from calculate.lib.cl_print import color_print
 from calculate.lib.cl_lang import setLocalTranslate
-setLocalTranslate('cl_core',sys.modules[__name__])
+setLocalTranslate('cl_core3',sys.modules[__name__])
+from itertools import *
 
 colorPrint = color_print()
 
+class BoolAction(argparse.Action):
+    reTrue = re.compile("^(?:y|ye|yes|t|tr|tru|true|on)$",re.I)
+    reFalse = re.compile("^(?:of|off|n|no|f|fa|fal|fals|false)$",re.I)
+
+    def __init__(self,
+                 option_strings,
+                 dest,
+                 nargs="?",
+                 const=None,
+                 default=None,
+                 type=None,
+                 choices=None,
+                 required=False,
+                 help=None,
+                 metavar=None):
+        super(BoolAction, self).__init__(
+            option_strings=option_strings,
+            dest=dest,
+            nargs=nargs,
+            const=const,
+            default=default,
+            type=type,
+            choices=choices,
+            required=required,
+            help=help,
+            metavar=metavar)
+
+    def __call__(self, parser, ns, values, option):
+        if values is None:
+            values = "on"
+        else:
+            if self.reTrue.match(values):
+                values = "on"
+            elif self.reFalse.match(values):
+                values = "off"
+            else:
+                msg = _('value may be on or off')
+                parser.error(msg)
+        setattr(ns, self.dest, values)
+
 def _print (*args):
     print " ".join(map(lambda x:unicode(x).encode('utf-8'),args))
 
-def get_password(text1 = None, text2 = None, getfromstdin=False):
+def get_password(text1 = None, text2 = None, getfromstdin=False,
+                 needrepeat=True):
     if getfromstdin:
         try:
             passwd = ''
@@ -47,6 +91,8 @@ def get_password(text1 = None, text2 = None, getfromstdin=False):
         pass2 = 'repeat'
         while pass1 != pass2:
             pass1 = getpass.getpass(text1)
+            if not needrepeat:
+                return pass1
             pass2 = getpass.getpass(text2)
             if pass1 != pass2:
                 print _('Passwords do not match')
@@ -78,27 +124,13 @@ def _getattr(obj, attr):
     return getattr(obj, attr) if hasattr(obj, attr) else None
 
 
-def get_terminal_size():
-    import struct
-    try:
-        import fcntl
-        import termios
-    except ImportError:
-        return (None, None)
-
-    try:
-        data = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, 4 * '00')
-        height, width = struct.unpack('4H', data)[:2]
-        return (height, width)
-    except Exception:
-        return (None, None)
-
 import argparse
 import textwrap as _textwrap
+from calculate.lib.utils.text import getTerminalSize
 class RawAndDefaultsHelpFormatter(argparse.HelpFormatter):
     def __init__(self, prog, max_help_position=24, **kwargs):
         # Use the whole terminal width
-        height, width = get_terminal_size()
+        height, width = getTerminalSize()
         argparse.HelpFormatter.__init__(self, prog, width=width,
                                         max_help_position = max_help_position,
                                         **kwargs)
@@ -136,22 +168,51 @@ def get_method_argparser(view, args, cl_core = False):
             elif field.opt:
                 opt = field.opt
                 data = {'dest':field.name,'help' : opt.help}
-                if field.type == "password":
+                if "choice" in field.type and hasattr(field.opt,"syntax") and \
+                    field.opt.syntax and "{" in field.opt.syntax:
+                    lgroup = group.add_mutually_exclusive_group()
+                    help = dict(map(lambda x:(x[0].strip("'"),x[2]),
+                                map(lambda x:x.partition(' - '),
+                                field.help.split(',\n'))))
+                    choice = field.choice
+                    if not type(choice) in (list,tuple):
+                        choice = choice.string
+                    for val in filter(None,choice):
+                        data['action'] = 'store_const'
+                        #data['nargs'] = '?'
+                        data['const'] = val
+                        data['metavar'] = ""
+                        data['help'] = help.get(val,field.help)
+                        lgroup.add_argument(field.opt.syntax.format(choice=val),
+                                            **data)
+                    continue
+                if "password" in field.type:
                     data['action'] = "store_true"
+                    if "need" in field.type:
+                        data['help'] = argparse.SUPPRESS
                 else:
                     data['type'] = str
+                if field.element in ['check', 'check_tristate']:
+                    data['action'] = BoolAction
+                elif field.element == 'radio' and field.type == 'bool':
+                    data['action'] = BoolAction
                 if field.element == 'table' and field.type != 'steps':
                     data['action'] = 'append'
 
                 if data.get('action') != "store_true":
                     data['metavar'] = opt.metavalue if opt.metavalue \
                                               else field.name.upper()
-                    if ':' in data['metavar']:
-                        data['metavar'] = field.name.upper()
+                    #if ':' in data['metavar']:
+                    #    data['metavar'] = field.name.upper()
+                if field.element in ['check']:
+                    data['metavar'] = "ON/OFF"
                 try:
                     opts = filter(None,[opt.shortopt,opt.longopt])
+                    if any(not "-" in x for x in opts):
+                        data.pop('dest')
+                        data['nargs'] = '?'
                     group.add_argument(*opts,**data)
-                except argparse.ArgumentError:
+                except argparse.ArgumentError as e:
                     continue
     if error_flag:
         raise Exception
@@ -180,7 +241,7 @@ def set_table_pwd(client, param_object, field, value):
         for column in range(len(ChoiceValues)):
             if found_flag:
                 break
-            if ChoiceValues[column].typefield == 'password':
+            if "password" in ChoiceValues[column].typefield:
                 if hasattr(field.tablevalue.body, 'stringArray'):
                     body = field.tablevalue.body.stringArray
                 else:
@@ -208,7 +269,7 @@ def set_table_pwd(client, param_object, field, value):
         for column in range(len(ChoiceValues)):
             if found_flag:
                 break
-            if ChoiceValues[column].typefield == 'password':
+            if "password" in ChoiceValues[column].typefield:
                 for _row in user_table:
                     if hasattr(_row, 'string'):
                         row = _row.string
@@ -228,7 +289,7 @@ def set_table_pwd(client, param_object, field, value):
 
     if client:
         if not getattr(param_object, field.name):
-            param_object[field.name] = field.tablevalue.body
+            setattr(param_object,field.name,field.tablevalue.body)
 
         result = []
         for _array in param_object[field.name].stringArray:
@@ -245,7 +306,9 @@ def set_table_pwd(client, param_object, field, value):
         return param_object
     else:
         if not getattr(param_object, field.name):
-            param_object[field.name] = field.tablevalue.body
+            setattr(param_object,field.name,
+                       map(lambda x: x[1:] if not x[0] else x,
+                       field.tablevalue.body))
 
         result = []
         for _array in getattr(param_object,field.name):
@@ -355,18 +418,18 @@ def collect_object(client, param_object, view, args,wait_thread = None,
             if field.element in ['check', 'check_tristate'] and \
                                                         field.type == 'bool':
                 set_flag = False
-                if hasattr (field.opt, 'metavalue'):
-                    if field.opt.metavalue and _getattr(args, field.name):
-                        if type(_getattr(args, field.name)) == bool:
-                            if _getattr(args, field.name) == True:
-                                param_object = set_obj_item(client,
-                                                    param_object, field.name,
-                                                    field.opt.metavalue)
-                        elif _getattr(args, field.name).lower() in \
-                                                                ['on','yes']:
-                            param_object = set_obj_item(client, param_object,
-                                             field.name, field.opt.metavalue)
-                            set_flag = True
+                #if hasattr (field.opt, 'metavalue'):
+                #    if field.opt.metavalue and _getattr(args, field.name):
+                #        if type(_getattr(args, field.name)) == bool:
+                #            if _getattr(args, field.name) == True:
+                #                param_object = set_obj_item(client,
+                #                                    param_object, field.name,
+                #                                    field.opt.metavalue)
+                #        elif _getattr(args, field.name).lower() in \
+                #                                                ['on','yes']:
+                #            param_object = set_obj_item(client, param_object,
+                #                             field.name, field.opt.metavalue)
+                #            set_flag = True
                 if not set_flag:
                     value = _getattr(args, field.name)
                     if value:
@@ -401,11 +464,13 @@ def collect_object(client, param_object, view, args,wait_thread = None,
                                    'file', 'radio']:
                 param_object = set_obj_item(client, param_object, field.name,
                                             _getattr(args, field.name))
-            elif field.element in ['password'] and _getattr(args, field.name):
+            elif field.element in ['password'] and _getattr(args, field.name) \
+                or field.type and  "need" in field.type:
                 if wait_thread:
                     wait_thread.pause()
                 password=get_password("%s: "%field.label,_('Repeat password: '),
-                                      getfromstdin=stdin_passwd)
+                                      getfromstdin=stdin_passwd,
+                                      needrepeat=not "one" in field.type)
                 if password is None:
                     raise KeyboardInterrupt
                 param_object = set_obj_item(client, param_object, field.name,
@@ -455,11 +520,49 @@ def collect_object(client, param_object, view, args,wait_thread = None,
 
     return param_object, steps
 
+def convertArgDictToList(argDict):
+    """
+    Convert list of dict like {'arg_1':None,'arg_2':'value2','arg_5':'value5'}
+    to [..,['','value2','','','value5']..] (iterator)
+    """
+    for row in argDict:
+        outrow = []
+        yield [row[i] or '' for i in sorted(row.keys())]
+
 def collect_table(field, val_list, client, wait_thread = None,
                   stdin_passwd = False):
     if not val_list:
         return None
-    val_table = map(lambda x: x.split(':'), val_list)
+    # if specified syntax for table row
+    if hasattr(field.opt,"syntax") and field.opt.syntax:
+        reArgs = re.compile(field.opt.syntax)
+        # check for correct sentense
+        for wrong in ifilterfalse(reArgs.search,
+                val_list):
+            # raise Error on wrong
+            raise ValueError(_("Wrong value syntax")+" "+wrong)
+        # create groupdict from all vals
+        argDict = map(lambda x:x.groupdict(),
+                  map(reArgs.search,
+                  val_list))
+        # convert groupdicts to val_table
+        val_table = list(convertArgDictToList(argDict))
+    # standard syntax
+    else:
+        val_table = map(lambda x: x.split(':'), val_list)
+    if "DEBUG" in os.environ:
+        print "VAL_TABLE",val_table
+
+    if type(field.tablevalue.values) in [list, tuple]:
+        choiceValue = field.tablevalue.values
+    else:
+        choiceValue = field.tablevalue.values.ChoiceValue
+    choiceValue = filter(lambda x:x.typefield != 'readonly',
+                  choiceValue)
+    lenChoiceValue = len(choiceValue)
+    for wrong in ifilter(lambda x:len(x)>lenChoiceValue,
+                 val_table):
+        raise ValueError(_("Wrong value syntax")+" "+wrong)
 
 #    obj_body = []
 
@@ -481,69 +584,44 @@ def collect_table(field, val_list, client, wait_thread = None,
 #
 #    obj_body = collect_obj_body(obj_body, field)
 
-    if type(field.tablevalue.values) in [list, tuple]:
-        ChoiceValue = field.tablevalue.values
-        column = len(field.tablevalue.head)
-    else:
-        ChoiceValue = field.tablevalue.values.ChoiceValue
-        column = len(field.tablevalue.head.string)
-    type_list = map(lambda x: x.typefield, ChoiceValue)
+    bGetPassword = any('password' in x.typefield for x in choiceValue)
     obj_body = []
-    for i in range(len(val_table)):
-        if 'password' in type_list:
-            if len(val_table[i]) != 2 or val_table[i][1].lower() != '':
+    for line in val_table:
+        receivedPassword = None
+        if bGetPassword:
+            if len(line) > 1 and line[1].lower() != '':
                 if wait_thread:
                     wait_thread.stop()
                     sys.stdout.write('\r')
                     sys.stdout.flush()
-                password=get_password(_('Password for %s: ')%val_table[i][0],\
-                               _('Repeat password for %s: ') %val_table[i][0],
+                receivedPassword=get_password(_('Password for %s: ')%line[0],\
+                                        _('Repeat password for %s: ') %line[0],
                                getfromstdin=stdin_passwd)
                 if password == None:
                     raise KeyboardInterrupt
         temp_row = []
-        for j in range(column):
-            # not adding if readonly
-            if j > (len(ChoiceValue) + 1):
-                continue
-
-            choice_value = ChoiceValue[j]
+        offset = 0
+        for val,choice_value in izip_longest(line,
+                        filter(lambda x:x.typefield != 'readonly',
+                        choiceValue),fillvalue=''):
             typefield = choice_value.typefield
             # if readonly, except first column
-
-            if typefield == 'readonly' and j > 0:
-                continue
-            elif typefield in ['check', 'check_tristate']:
-                if len (val_table[i]) < j + 1:
-                    temp_row.append('')
-                    continue
-                if val_table[i][j].lower() in ['on', 'yes']:
+            if typefield in ['check', 'check_tristate']:
+                if BoolAction.reTrue.match(val):
                     temp_row.append('on')
-                elif val_table[i][j].lower() in ['off', 'no']:
+                elif BoolAction.reFalse.match(val):
                     temp_row.append('off')
                 else:
-                    temp_row.append(val_table[i][j])
+                    temp_row.append(val)
             elif typefield in ['input', 'combo', 'comboEdit', 'openfile', \
-                               'file', 'radio', 'text']:
-                if len (val_table[i]) < j + 1:
-                    temp_row.append('')
-                else:
-                    temp_row.append(val_table[i][j])
+                               'file', 'radio', 'text', 'multichoice',
+                               'multichoice_add']:
+                temp_row.append(val)
             elif typefield == 'password':
-                if 'password' in locals():
-                    temp_row.append(password)
-                    del password
+                if not receivedPassword is None:
+                    temp_row.append(receivedPassword)
                 else:
-                    if len (val_table[i]) < j + 1:
-                        temp_row.append('')
-                    else:
-                        temp_row.append(val_table[i][j])
-            elif typefield in ['multichoice', 'multichoice_add']:
-                if len (val_table[i]) < j + 1:
-                    temp_row.append('')
-                else:
-                    temp_row.append(val_table[i][j])
-
+                    temp_row.append(val)
         obj_body.append(temp_row)
 
     if not obj_body:
@@ -597,3 +675,28 @@ def collect_obj_body(body, field):
                     temp_row.append(body[i][j])
         result_table.append(temp_row)
     return result_table
+
+def getErrorOnParam(args,field):
+    """
+    Get errors for param
+    """
+    params_text = ""
+    if any("-" in x 
+        for x in filter(None,(field.opt.longopt,field.opt.shortopt))):
+        paramName = ', '.join(filter(None,
+                [field.opt.shortopt, field.opt.longopt]))
+    else:
+        paramName = field.opt.metavalue
+    if getattr(args,field.name) is None and \
+        not "need" in field.type:
+        params_text += _("Need to specify the parameter ")
+        params_text += paramName+'. '
+    else:
+        if "need" in field.type:
+            params_text += \
+                _('Error in field \'%s\'. ')%field.label
+        else:
+            params_text += _('Error in parameter ')
+            params_text += paramName +'. '
+    return params_text
+
