@@ -19,36 +19,53 @@ from methods_func import get_method_argparser, collect_object, \
                          check_result_msg, get_param_pwd, _print
 from api_types import ViewInfo
 from calculate.core.client.progressbar import Bar,Percentage,ETA,ProgressBar
+from calculate.lib.cl_progressbar import ClProgressDialog,ClMessageBox
+from calculate.lib.utils.text import tableReport
 from cert_cmd import parse
 import sys, termios
+import os
+import re
+from methods_func import getErrorOnParam
+from func import shortTraceback
 from fcntl import ioctl
 from array import array
+import threading
 from calculate.lib.cl_print import color_print
 from calculate.lib.datavars import VariableError
 from calculate.lib.cl_lang import setLocalTranslate
-setLocalTranslate('cl_core',sys.modules[__name__])
+setLocalTranslate('cl_core3',sys.modules[__name__])
+from itertools import repeat
 
 class replaceClass():
+    method_status = 2
     def startprocess (self, sid, target=None, method=None, method_name=None, \
                       auto_delete=False, args_proc = {}):
         """ start process """
-        com = target(self.no_progress)
+        if "LANG" in os.environ:
+            curThread = threading.currentThread()
+            curThread.lang = os.environ["LANG"]
+        com = target(self.no_progress,self.gui_progress,self.gui_warning)
         if hasattr (com.__class__.__bases__[1], '__init__'):
             com.__class__.__bases__[1].__init__(com)
-        getattr(com, method)(*args_proc)
+        self.method_status = 0 if getattr(com, method)(*args_proc) else 1
         return 0
 
     class Common:
         """ class to interact with the processes """
-        def __init__(self, no_progress):
+        reClearTags = re.compile("<.*?>")
+        def __init__(self, no_progress, gui_progress,gui_warning):
             self.pid = 0
             self.Num = 100000
             self.color_print = color_print()
 
             self.no_progress = no_progress
+            self.gui_progress = gui_progress
+            self.gui_warning = gui_warning
             self.progress = None
             self.progressbar = None
             self.last_progress_msg = ''
+            self.error = []
+            self.warning = []
 
         def writeFile(self):
             pass
@@ -59,7 +76,8 @@ class replaceClass():
                 return
             if perc >= 100 or perc < 0:
                 if self.progressbar:
-                    self.progressbar.finish()
+                    if not self.gui_progress:
+                        self.progressbar.finish()
                 return
             self.last_progress_msg = self.print_progressbar(perc,
                                             short_message, long_message,
@@ -67,22 +85,47 @@ class replaceClass():
 
         def print_progressbar(self, percent, short_message, long_message,
                               last_msg = None, error = False):
+            if not self.progressbar:
+                return
             if long_message:
-                if last_msg != long_message:
-                    cout_progress()
-                    for line in long_message.splitlines():
-                        self.color_print.printSUCCESS(line)
-                self.progressbar.update(percent)
+                if self.gui_progress:
+                    self.progressbar.setTextLabel(long_message)
+                    self.progressbar.setMaximum(100)
+                    self.progressbar.setTextVisible(True)
+                    if percent == 100:
+                        self.progressbar.setValue(99)
+                    self.progressbar.setValue(percent)
+                else:
+                    if last_msg != long_message:
+                        cout_progress()
+                        for line in long_message.splitlines():
+                            self.color_print.printSUCCESS(line)
+                    self.progressbar.update(percent)
                 return long_message
             elif short_message:
-                if last_msg != short_message:
-                    cout_progress()
-                    for line in short_message.splitlines():
-                        self.color_print.printSUCCESS(line)
-                self.progressbar.update(percent)
+                if self.gui_progress:
+                    self.progressbar.setTextLabel(short_message)
+                    self.progressbar.setMaximum(100)
+                    self.progressbar.setTextVisible(True)
+                    if percent == 100:
+                        self.progressbar.setValue(99)
+                    self.progressbar.setValue(percent)
+                else:
+                    if last_msg != short_message:
+                        cout_progress()
+                        for line in short_message.splitlines():
+                            self.color_print.printSUCCESS(line)
+                    self.progressbar.update(percent)
                 return short_message
             else:
-                self.progressbar.update(percent)
+                if self.gui_progress:
+                    self.progressbar.setMaximum(100)
+                    self.progressbar.setTextVisible(True)
+                    if percent == 100:
+                        self.progressbar.setValue(99)
+                    self.progressbar.setValue(percent)
+                else:
+                    self.progressbar.update(percent)
                 return last_msg
 
         def setStatus(self, stat):
@@ -98,17 +141,35 @@ class replaceClass():
             if self.no_progress:
                 return self.progress
             if self.progressbar:
-                return self.progressbar.currval
+                if self.gui_progress:
+                    return self.progressbar.value()
+                else:
+                    return self.progressbar.currval
             return 100
 
-        def addProgress(self):
+        def addProgress(self,message):
             if self.no_progress:
                 self.progress = 0
                 return
-            widgets = ['','', Bar(), '', Percentage(),' ', ETA()]
-            self.progressbar = ProgressBar(widgets=widgets, maxval=100)
-            self.progressbar.start()
-            self.progress = 0
+            if self.gui_progress:
+                if ClProgressDialog:
+                    if not self.progressbar:
+                        self.progressbar = ClProgressDialog()
+                        self.progressbar.setCancelButton(None)
+                        self.progressbar.adjustSize()
+                        self.progressbar.setWindowTitle("")
+                        self.progressbar.setAutoClose(False)
+                        self.progressbar.setAutoReset(False)
+                    self.progressbar.setLabelText(message.decode('utf-8'))
+                    self.progressbar.setMaximum(0)
+                    self.progressbar.setTextVisible(False)
+                else:
+                    self.progressbar = None
+            else:
+                widgets = ['','', Bar(), '', Percentage(),' ', ETA()]
+                self.progressbar = ProgressBar(widgets=widgets, maxval=100)
+                self.progressbar.start()
+                self.progress = 0
             self.addMessage(type = 'progress', id = id)
 
         def printTable(self, table_name, head, body, fields = None,\
@@ -119,28 +180,68 @@ class replaceClass():
         def addMessage(self, type = 'normal', message = None, id = None):
             self.printSUCCESS(message)
 
+        def printDefault(self,message = ''):
+            message = self.clearTags(message)
+            self.color_print.defaultPrint(message+"\n")
+
+        def clearTags(self,s):
+            """
+            Remove tags from text
+            """
+            reColor = re.compile("color=['\"](.*?)['\"]")
+            def fontFunc(part):
+                attr,text = part.groups()
+                color = reColor.search(attr)
+                if color:
+                    color = color.group(1)
+                    if "light" in color or "dark gray" in color:
+                        bg = "1"
+                    else:
+                        bg = "0"
+                    color = color.replace("light","").strip()
+                    colorMap = {'red':"31",
+                                'green':"32",
+                                'brown':"33",
+                                'blue':"34",
+                                'purple':"35",
+                                'cyan':"36",
+                                'gray':"37",
+                                'dark gray':"30"}
+                    if color in colorMap:
+                        fg = colorMap[color]
+                        return "\033[%s;%sm%s\033[0m"%(bg,fg,text)
+                return text
+            reFont = re.compile("<font (.*?)>(.*?)</font>")
+            return self.reClearTags.sub("",reFont.sub(fontFunc,s))
+            #return self.reClearTags.sub("",s)
+
         def printSUCCESS(self, message = '', type = None, id = None):
             if message:
-                if self.progressbar:
+                if self.progressbar and not self.gui_progress:
                     if not self.progressbar.finished:
                         cout_progress()
                         for line in message.splitlines():
-                            self.color_print.printSUCCESS(line)
+                            self.color_print.printSUCCESS(self.clearTags(line))
                         return
                 if message:
                     for line in message.splitlines():
-                        self.color_print.printSUCCESS(line)
+                        self.color_print.printSUCCESS(self.clearTags(line))
 
         def printWARNING(self, message):
-            if self.progressbar and not self.no_progress:
+            if self.progressbar and not self.no_progress \
+                    and not self.gui_progress:
                 if not self.progressbar.finished:
                     cout_progress()
                     for line in message.splitlines():
+                        line = self.clearTags(line)
                         self.color_print.printWARNING(line)
                     return
             if message:
                 for line in message.splitlines():
+                    line = self.clearTags(line)
                     self.color_print.printWARNING(line)
+            if message:
+                self.warning.append(message)
 
         def printERROR(self, message = ''):
             perc = self.getProgress()
@@ -154,19 +255,22 @@ class replaceClass():
                 self.setProgress(perc)
             if message:
                 for line in message.splitlines():
+                    line = self.clearTags(line)
                     self.color_print.printERROR(line)
+            if message:
+                self.error.append(message)
 
         def startTask(self, message, progress = False, num = 1):
             self.printSUCCESS(message = message)
             if progress:
-                self.addProgress()
+                self.addProgress(message)
 
         def setTaskNumber(self, number = None):
             pass
 
         def endTask(self, result = None, progress_message = None):
             if result:
-                self.color_print.printOK(result)
+                self.color_print.printOK(self.clearTags(result))
             self.setProgress(100, progress_message)
 
         def askQuestion(self, message):
@@ -174,7 +278,9 @@ class replaceClass():
 
         def askPassword(self, message, twice = False):
             from calculate.lib.utils.common import getpass
-            text1 = message
+            if os.readlink('/proc/self/fd/0') == '/dev/console':
+                os.system('chvt 1 &>/dev/null')
+            text1 = "%s:"%message
             if not twice:
                 return getpass.getpass(text1)
             if twice:
@@ -197,6 +303,16 @@ class replaceClass():
 
         def endFrame(self):
             self.printSUCCESS(type = 'endFrame')
+            if self.gui_progress and self.progressbar:
+                self.progressbar.quit()
+            if self.warning and not self.error and self.gui_warning:
+                ClMessageBox().warning(("\n".join(self.warning)).decode('utf-8'))
+            if self.error and self.gui_progress:
+                if self.warning and self.gui_warning:
+                    error = "\n".join(self.warning+self.error)
+                else:
+                    error = "\n".join(self.error)
+                ClMessageBox().critical(error.decode('utf-8'))
 
         def startGroup(self, message):
             self.printSUCCESS(type = 'startGroup', message = message)
@@ -219,7 +335,7 @@ def cout_progress(string = None):
         sys.stdout.write('\r')
     sys.stdout.flush()
 
-def local_method(metaclass, args):
+def local_method(metaclass, args, unknown_args):
     """
     Call method from metaclass, check method existing.
 
@@ -231,7 +347,7 @@ def local_method(metaclass, args):
         if sym_link in metaclass.Dec.conMethods.keys():
             args.method = metaclass.Dec.conMethods[sym_link][0]
         else:
-            _print (_('Method for %s not found') %sym_link)
+            _print (_("Method not found for %s") %sym_link)
             sys.exit(1)
 
     if args.list_methods:
@@ -239,7 +355,7 @@ def local_method(metaclass, args):
         keys.sort()
         for key in keys:
             print metaclass.Dec.conMethods[key][0], ' - ', \
-                  metaclass.Dec.conMethods[key][1]
+                  metaclass.Dec.conMethods[key][2]
         return 0
 
     colorPrint = color_print()
@@ -251,6 +367,7 @@ def local_method(metaclass, args):
         view_obj.step = None
         view_obj.expert = True
         view_obj.brief = None
+        view_obj.onlyhelp = True
         try:
             view = getattr(metaObject, method_view_name)(0, view_obj)
         except AttributeError:
@@ -259,20 +376,29 @@ def local_method(metaclass, args):
         try:
             method_parser = get_method_argparser(view, args, cl_core = True)
         except Exception as e:
+            import traceback
+            for i in apply(traceback.format_exception, sys.exc_info()):
+                sys.stderr.write(i)
+            sys.stderr.flush()
             metaObject.clear_cache(0, method_name)
             return  1
         method_parser.print_help()
     else:
         try:
-            call_method(metaObject, args, colorPrint)
+            call_method(metaObject, args, unknown_args, colorPrint)
+            metaObject.clear_cache(0, method_name)
+            return metaObject.method_status
+        except (ValueError) as e:
+            colorPrint.printERROR(shortTraceback(*sys.exc_info()))
         except (KeyboardInterrupt, EOFError):
-            colorPrint.printERROR(_('Interrupted by the user'))
-        except Exception:
+            colorPrint.printERROR(_('Manually interrupted'))
+        except Exception as e:
+            colorPrint.printERROR(shortTraceback(*sys.exc_info()))
             pass
 #            print 'Error: ', e
     metaObject.clear_cache(0, method_name)
 
-def call_method(metaObject, args, colorPrint):
+def call_method(metaObject, args, unknown_args, colorPrint):
     """
     Function for call method through metaObject and args
     """
@@ -280,10 +406,13 @@ def call_method(metaObject, args, colorPrint):
     stdin_passwd = args.stdin_passwd
     method_view_name = method_name + '_view'
     metaObject.no_progress = args.no_progress
+    metaObject.gui_progress = args.gui_progress
+    metaObject.gui_warning = args.gui_warning
     view_obj = ViewInfo()
     view_obj.step = None
     view_obj.expert = True
     view_obj.brief = None
+    view_obj.onlyhelp = False
 
     no_questions = args.no_questions
     try:
@@ -294,14 +423,23 @@ def call_method(metaObject, args, colorPrint):
     method_parser = get_method_argparser(view, args, cl_core = True)
     param_object = create_param_object(view)
     try:
-        args, unknown_args = method_parser.parse_known_args()
-    except SystemExit:
+        args, unknown_args = method_parser.parse_known_args(unknown_args)
+    except SystemExit as e:
         return 1
+    except Exception as e:
+        import traceback
+        for i in apply(traceback.format_exception, sys.exc_info()):
+            sys.stderr.write(i)
+        sys.stderr.flush()
+        raise
     for i in unknown_args:
         if i.startswith('-'):
-            if i in parse().parse_known_args()[1]:
+            if i in parse(True).parse_known_args()[1]:
                 _print (_('Unknown parameter'), i)
                 return 1
+        else:
+            _print (_('Unknown argument'), i)
+            return 1
 
     param_object, steps = collect_object(None, param_object, view, args,
                                          stdin_passwd=stdin_passwd)
@@ -329,6 +467,7 @@ def call_method(metaObject, args, colorPrint):
         view_obj.step = None
         view_obj.expert = True
         view_obj.brief = True
+        view_obj.onlyhelp = False
         try:
             view = getattr(metaObject, method_view_name)(0, view_obj)
         except AttributeError:
@@ -343,7 +482,7 @@ def call_method(metaObject, args, colorPrint):
                     ask = 'no'
                     print
                 if ask.lower() in ['n', 'no']:
-                    colorPrint.printERROR(_('Interrupted by the user'))
+                    colorPrint.printERROR(_('Manually interrupted'))
                     return None
                 if ask.lower() in ['y', 'yes']:
                     break
@@ -363,12 +502,9 @@ def call_method(metaObject, args, colorPrint):
             for Group in view.groups:
                 for field in Group.fields:
                     if field.name == ReturnedMessage.field:
-                        if field.opt.shortopt or field.opt.longopt:
-                            params_text += _('Error in parameter ')
-                            params_text += ', '.join(filter(None,
-                                [field.opt.shortopt, field.opt.longopt]))+'. '
+                        params_text += getErrorOnParam(args,field)
             colorPrint.printERROR('\r' + params_text + \
-                                         ReturnedMessage.message)
+                                     ReturnedMessage.message)
             return None
     return method_result
 
@@ -501,7 +637,7 @@ def print_brief_group(Fields, group_name):
                 body = [['']*len(head)]
                 res = printTable(body, head)
                 sys.stdout.flush()
-                sys.stdout.write(res)
+                sys.stdout.write(res+"\n")
                 continue
             ChoiceValue = field.tablevalue.values
             for row in xrange(len(ChoiceValue)):
@@ -513,7 +649,7 @@ def print_brief_group(Fields, group_name):
                             body[i][row] = _('no')
                         if body[i][row] == 'auto':
                             body[i][row] = _('auto')
-                if ChoiceValue[row].typefield == 'password':
+                if "password" in ChoiceValue[row].typefield:
                     for i in xrange(len(body)):
                         if body[i][row]:
                             body[i][row] = '***'
@@ -527,46 +663,29 @@ def print_brief_group(Fields, group_name):
             colorPrint.printSUCCESS(field.label+': ')
             res = printTable(data, head)
             sys.stdout.flush()
-            sys.stdout.write(res)
+            sys.stdout.write(res+"\n")
         else:
             uncompatible_count += 1
 
 #    if uncompatible_count == len (Fields) and group_name:
 #        colorPrint.printSUCCESS(_('Not used'))
 
-def printTable(data, header=None ):
-    res = []
-    for row in data:
-        encode_row = map(lambda x: x.encode('utf-8') if x else '',row)
-        res.append(encode_row)
-    data = res
+class Table(tableReport):
+    def __init__(self,*args,**kwargs):
+        self.res = []
+        tableReport.__init__(self,*args,**kwargs)
 
-    if type(header) == list:
-        header = map(lambda x: x.encode('utf-8'), header)
+    def printFunc(self,s):
+        self.res.append(s)
 
-    lens = [0]*len(data[0])
-    for row in data:
-        for numCol, column in enumerate(row):
-            cLen = len(unicode(str(column).decode('utf-8')))
-            if lens[numCol] < cLen:
-                lens[numCol] = cLen
-    res = ""
-    spacer = lambda lenList: '+' + \
-                          '+'.join( '-'*(x+2) for x in lenList) + '+'
-    drawRow = lambda lenList, valueList: '|' + "".join(
-            ' %s ' % item + ' ' * (lenList[num] - \
-            len(unicode(str(item).decode('utf-8')))) + '|'
-            for num, item in enumerate(valueList) )
-    if header:
-        for numCol, column in enumerate(header):
-            cLen = len(unicode(str(column).decode('utf-8')))
-            if lens[numCol] < cLen:
-                lens[numCol] = cLen
-        res += spacer(lens)+ "\n"
-        res += drawRow( lens, header )+ "\n"
+    def printTable(self):
+        self.setAutosize()
+        self.printReport(printRows=False)
+        return "\n".join(self.res)
 
-    res+= spacer(lens) + "\n"
-    for r in data:
-        res += drawRow( lens, r )+ "\n"
-    res+= spacer(lens)+ "\n"
-    return res
+def printTable(data, header=None):
+    try:
+        return Table(None,header,data,colSpan=0).printTable()
+    except Exception as e:
+        print str(e)
+        raise
